@@ -5,7 +5,7 @@ use anyhow::{anyhow, bail, Context, Result};
 use clap::Parser;
 
 use super::{progress_counter, RusticConfig};
-use crate::backend::{LocalBackend, LocalSource, LocalSourceOptions};
+use crate::backend::{LocalDestination, LocalSource, LocalSourceOptions};
 use crate::blob::{Node, NodeStreamer, NodeType, Tree};
 use crate::commands::helpers::progress_spinner;
 use crate::crypto::hash;
@@ -79,7 +79,7 @@ pub(super) fn execute(
 
             let index = IndexBackend::new(be, progress_counter(""))?;
             let node1 = Tree::node_from_path(&index, snap1.tree, Path::new(path1))?;
-            let local = LocalBackend::new(path2)?;
+            let local = LocalDestination::new(path2, false, !node1.is_dir())?;
             let path2 = PathBuf::from(path2);
             let is_dir = path2
                 .metadata()
@@ -123,7 +123,7 @@ fn arg_to_snap_path<'a>(arg: &'a str, default_path: &'a str) -> (Option<&'a str>
 }
 
 fn identical_content_local(
-    local: &LocalBackend,
+    local: &LocalDestination,
     index: &impl ReadIndex,
     path: &Path,
     node: &Node,
@@ -183,12 +183,12 @@ fn diff(
                 let node1 = &i1.1;
                 let node2 = &i2.1;
                 match node1.node_type() {
-                    tpe if tpe != node2.node_type() => println!("T    {:?}", path), // type was changed
+                    tpe if tpe != node2.node_type() => println!("T    {path:?}"), // type was changed
                     NodeType::File if !no_content && !file_identical(path, node1, node2)? => {
-                        println!("M    {:?}", path);
+                        println!("M    {path:?}");
                     }
                     NodeType::File if metadata && node1.meta() != node2.meta() => {
-                        println!("U    {:?}", path);
+                        println!("U    {path:?}");
                     }
                     NodeType::Symlink { linktarget } => {
                         if let NodeType::Symlink {
@@ -196,7 +196,7 @@ fn diff(
                         } = node2.node_type()
                         {
                             if *linktarget != *linktarget2 {
-                                println!("U    {:?}", path);
+                                println!("U    {path:?}");
                             }
                         }
                     }
